unit KnownIdents;

interface

uses
  SysUtils, Classes;

var
  UnknownIdentsList: TStringList;
  KnownIdentsList: TStringList;

function KnownIdent(s: string): string;
procedure InitLists(filename: string);
procedure SaveLists;

implementation

var
  known_filename: string;
  unknown_filename: string;

function KnownIdent(s: string): string;
var
  x: string;
  e: integer;
  num: integer;
begin
  (*      -  *)
  if s = '' then
    begin
      Result := '';
      exit;
    end;
  (*     '*' -   
          *)
  if s[Length(s)] = '*' then
    begin
      Result := KnownIdent('P'+Copy(s, 1, Length(s)-1));
      Exit;
    end;
  (*   .  ,      *)
  Val(s, num, e);
  if num = 0 then ;
  if (e = 0) and (Copy(s, 1, 2) <> '0x') then begin
    (*     .  . *)
    Result := s;
    exit;
  end;
  (*     . *)
  x := KnownIdentsList.Values[s];
  if x <> '' then
    begin
      (*  - ,  ,  . *)
      Result := x;
    end
  else
    begin
      (*    ,  ,   . *)
      Result := s;
      UnknownIdentsList.Add(s + '=');
    end;
end;

procedure InitLists(filename: string);
begin
  known_filename := changefileext(filename, '.known');
  unknown_filename := changefileext(filename, '.unknown');
  if FileExists(known_filename) then KnownIdentsList.LoadFromFile(known_filename);
end;

procedure SaveLists;
begin
  UnknownIdentsList.SaveToFile(unknown_filename);
end;

initialization
  UnknownIdentsList := TStringList.Create;
  KnownIdentsList := TStringList.Create;

  UnknownIdentsList.Duplicates := dupIgnore;
  UnknownIdentsList.Sorted := True;
finalization
  UnknownIdentsList.Free;
  KnownIdentsList.Free;

end.