unit Nodes2;

interface

uses
  SysUtils, Classes, LibXmlParser;

type
  Node = class(TObject)
    (*    .  -   . *)
    text: TStringList;
    (*   ,    *)
    proc: TStringList;
    (*  parent'   ,      
       xml  *)
    parent: Node;
    name, cname, typ: string;
    cursl: TStringList;
    buffer: string;
    needflush: boolean;
    procedure FlushBuffer;
    procedure AppendTextTo(sl: TStringList);
    procedure AppendProcTo(sl: TStringList);
    procedure SwitchTo(sl: TStringList);
    function Lookup(symbol: string): string;
    procedure SymbolToTable(symbol: string);
    procedure DoEmitCdecl;
    procedure Emit(s: string);
    procedure EmitLn(s: string);
    procedure EmitLnBoth(s: string);
    procedure EmitBoth(s: string);
    procedure Ln;

    procedure GenerateText; virtual;

    function boolvalue(name: string; a: TAttrList): boolean;
    function strvalue(name: string; a: TAttrList): string;
    procedure AddChild(n: Node); virtual; abstract;
    procedure parseAttr(a: TAttrList); virtual;
    constructor Create(aParent: Node); virtual;
    destructor Destroy; override;
  end;

  Alias = class(Node)
    procedure AddChild(n: Node); override;
    constructor Create(aParent: Node); override;
    procedure GenerateText; override;
  end;

  Boxed = class(Node)
    hidden, opaque: boolean;
    fields, methods,
    constructors: TObjectList;
    maxfieldlength: integer;
    procedure AddChild(n: Node); override;
    procedure parseAttr(a: TAttrList); override;
    constructor Create(aParent: Node); override;
    destructor Destroy; override;
    procedure GenerateText; override;
  end;

  ReturnType = class(Node)
    element_type: string;
    elements_owned: boolean;
    owned: boolean;
    procedure AddChild(n: Node); override;
    procedure parseAttr(a: TAttrList); override;
  end;

  Parameter = class(Node)
    pass_as: string;
    property_name: string;
    scope: string;
    isarray: boolean;
    ellipsis: boolean;
    printf_format_args: boolean;
    owned: boolean;
    printf_format: boolean;
    procedure AddChild(n: Node); override;
    procedure parseAttr(a: TAttrList); override;
  end;

  Parameters = class(Node)
    paramlist: TObjectList;
    procedure AddChild(n: Node); override;
    constructor Create(aParent: Node); override;
    destructor Destroy; override;
  end;

  Callback = class(Node)
    rettype: ReturnType;
    params: Parameters;
    procedure AddChild(n: Node); override;
    procedure GenerateText; override;
    procedure GenerateCallbackDef(maxlength: integer);
  end;

  Member = class(Node)
    value: string;
    procedure AddChild(n: Node); override;
    procedure parseAttr(a: TAttrList); override;
    procedure GenerateText; override;
  end;

  Enum = class(Node)
    _deprecated: boolean;
    gtype: string;
    hidden: boolean;
    memberlist: TObjectList;
    maxmemberlength: integer;
    procedure AddChild(n: Node); override;
    procedure parseAttr(a: TAttrList); override;
    constructor Create(aParent: Node); override;
    destructor Destroy; override;
    procedure GenerateText; override;
  end;

  Method = class(Node)
    _deprecated: boolean;
    hidden: boolean;
    shared: boolean;
    needs_ref: boolean;
    rettype: ReturnType;
    params: Parameters;
    procedure parseAttr(a: TAttrList); override;
    procedure AddChild(n: Node); override;
    procedure GenerateText; override;
  end;

  Signal = class(Node)
    hidden: boolean;
    when: string;
    rettype: ReturnType;
    params: Parameters;
    procedure parseAttr(a: TAttrList); override;
    procedure AddChild(n: Node); override;
    procedure GenerateText; override;
  end;

  VMethod = class(Node)
    rettype: ReturnType;
    params: Parameters;
    procedure AddChild(n: Node); override;
  end;

  _Property = class(Node)
    construct_only: boolean;
    readable: boolean;
    writeable: boolean;
    construct: boolean;
    _deprecated: boolean;
    hidden: boolean;
    new_flag: boolean;
    procedure AddChild(n: Node); override;
    procedure parseAttr(a: TAttrList); override;
  end;

  _Interface = class(Node)
    properties,
    signallist,
    methodlist,
    vmethodlist: TObjectList;
    procedure AddChild(n: Node); override;
    constructor Create(aParent: Node); override;
    destructor Destroy; override;
    procedure GenerateText; override;
  end;

  Field = class(Node)
    access: string;
    _array: boolean;
    array_len: string;
    hidden: boolean;
    bits: string;
    writeable: boolean;
    new_flag: boolean;
    procedure parseAttr(a: TAttrList); override;
    procedure AddChild(n: Node); override;
  end;

  _Constructor = class(Node)
    _deprecated,
    hidden,
    preferred,
    shared: boolean;
    rettype: ReturnType;
    params: Parameters;
    procedure parseAttr(a: TAttrList); override;
    procedure AddChild(n: Node); override;
    constructor Create(aParent: Node); override;
    destructor Destroy; override;
    procedure GenerateText; override;
  end;

  Implements = class(Node)
    interfaces: TObjectList;
    procedure AddChild(n: Node); override;
    constructor Create(aParent: Node); override;
    destructor Destroy; override;
  end;

  Childprop = class(Node)
    readable: boolean;
    writeable: boolean;
    procedure parseAttr(a: TAttrList); override;
    procedure AddChild(n: Node); override;
  end;

  StaticString = class(Node)
    value: string;
    procedure parseAttr(a: TAttrList); override;
    procedure AddChild(n: Node); override;
  end;

  _Object = class(Node)
    objparent: string;
    _deprecated: boolean;
    disable_gtype_ctor: boolean;
    fields,
    properties,
    signals,
    methods,
    vmethods,
    childprops,
    staticstrings,
    constructors: TObjectList;
    implement: Implements;
    children: TObjectList;
    nls: array of integer;
    {  AddChild  ,    .
        ,    ,    
        (    ). }
    maxfieldlength: integer;
    procedure AddExtender(n: Node);
    procedure SortChildren;
    procedure parseAttr(a: TAttrList); override;
    procedure AddChild(n: Node); override;
    constructor Create(aParent: Node); override;
    destructor Destroy; override;

    procedure GenerateText; override;
  end;

  Struct = class(Node)
    constructors,
    methods,
    callbacks,
    fields: TObjectList;
    _deprecated: boolean;
    hidden: boolean;
    opaque: boolean;
    maxfieldlength: integer;
    procedure parseAttr(a: TAttrList); override;
    procedure AddChild(n: Node); override;
    constructor Create(aParent: Node); override;
    destructor Destroy; override;
    procedure GenerateText; override;
  end;

  _Class = class(Node)
    methods: TObjectList;
    hidden: boolean;
    procedure parseAttr(a: TAttrList); override;
    procedure AddChild(n: Node); override;
    constructor Create(aParent: Node); override;
    destructor Destroy; override;
  end;

  Symbol = class(Node)
    procedure AddChild(n: Node); override;
  end;

  Namespace = class(Node)
    alibrary: string;
    aliases: TObjectList;
    enums: TObjectList;
    objects: TObjectList;
    classes: TObjectList;
    structs: TObjectList;
    callbacs: TObjectList;
    interfaces: TObjectList;
    boxeds: TObjectList;
    maxobjectlength: integer;
    maxstructlength: integer;
    maxaliaslength: integer;
    maxboxedlength: integer;
    maxcallbacklength: integer;
    maxinterfacelength: integer;
    constructor Create(aParent: Node); override;
    procedure AddChild(n: Node); override;
    destructor Destroy; override;
    procedure parseAttr(a: TAttrList); override;

    procedure GenerateText; override;
  end;

  Api = class(Node)
    ns: Namespace;
    namespaces: TObjectList;
    symbols: TObjectList;
    constructor Create(aParent: Node); override;
    destructor Destroy; override;
    procedure AddChild(n: Node); override;

    procedure GenerateText; override;
  end;

implementation

uses
  Utils, Contnrs, Variants, Settings, SymbolTables;

{ Node }

function Node.boolvalue(name: string; a: TAttrList): boolean;
begin
  Result := (a.Value(name) = '1') or (a.Value(name) = 'true');
end;

constructor Node.Create(aParent: Node);
begin
  parent := aParent;
  (*   ,       Emit *)
  text := TStringList.Create;
  //text.Add('');
  proc := TStringList.Create;
  //proc.Add('');
  cursl := text;
  needflush := false;
end;

destructor Node.Destroy;
begin
  text.Free;
  proc.Free;
  inherited;
end;

procedure Node.parseAttr(a: TAttrList);
begin
  name  := strvalue('name', a);
  cname := strvalue('cname', a);
  typ   := strvalue('type', a);
end;

function Node.strvalue(name: string; a: TAttrList): string;
begin
  Result := a.Value(name);
end;

procedure Node.Emit(s: string);
begin
  //if cursl.Count = 0 then cursl.Add('');
  //cursl[cursl.Count-1] := cursl[cursl.Count-1] + s;
  needflush := true;
  buffer := buffer + s;
end;

procedure Node.EmitLn(s: string);
begin
  Emit(s);
  Ln;
end;

procedure Node.Ln;
begin
  cursl.Add(buffer);
  buffer := '';
end;

procedure Node.GenerateText;
begin

end;

procedure Node.EmitLnBoth(s: string);
var
  old: TStringList;
begin
  old := cursl;
  cursl := text;
  EmitLn(s);
  cursl := proc;
  EmitLn(s);
  cursl := old;
end;

procedure Node.EmitBoth(s: string);
var
  old: TStringList;
begin
  old := cursl;
  cursl := text;
  Emit(s);
  cursl := proc;
  Emit(s);
  cursl := old;
end;

procedure Node.SwitchTo(sl: TStringList);
begin
  FlushBuffer;
  cursl := sl;
end;

procedure Node.FlushBuffer;
begin
  if needflush then begin
    cursl.Add(buffer);
    needflush := false;
  end;
end;

procedure Node.AppendProcTo(sl: TStringList);
begin
  FlushBuffer;
  if proc.Count > 0 then
    if proc[proc.Count-1] = '' then
      proc.Delete(proc.Count-1);
  sl.AddStrings(proc);
end;

procedure Node.AppendTextTo(sl: TStringList);
begin
  FlushBuffer;
  if text.Count > 0 then
    if text[text.Count-1] = '' then
      text.Delete(text.Count-1);
  sl.AddStrings(text);
end;

procedure Node.DoEmitCdecl;
begin
  if anSettings.show_cdecl then begin
    if anSettings.target_language = tlCP then begin
      Emit('[ccall]');
    end else begin
      Emit('["C"]');
    end;
  end;
end;

function Node.Lookup(symbol: string): string;
var
  i: integer;
  e: integer;
begin
  if symbol = '' then begin
    result := '';
    exit;
  end;

  if Copy(symbol, 1, 6) = 'const-' then begin
    symbol := Copy(symbol, 7, MaxInt);
  end;

  if symbol[Length(symbol)] = '*' then begin
    symbol := 'P' + Copy(symbol, 1, Length(symbol) -1);
  end;
  //    
  result := stlst.Lookup(symbol);
  //   - 
  if result <> '' then exit;

  // ,   .  ,  .
  val(symbol, i, e);
  { ? }
  if e = 0 then begin
    Result := symbol;
    Exit;
    // suppress warning
    if i = 0 then ;
  end;
end;

procedure Node.SymbolToTable(symbol: string);
begin
  curst.AddSymbol(symbol);
end;

{ Alias }

procedure Alias.AddChild(n: Node);
begin
  (* do nothing *)
end;

constructor Alias.Create(aParent: Node);
begin
  inherited;
end;

procedure Alias.GenerateText;
begin
  Emit(Ident + cname + '* = ' + Lookup(typ) + ';');
  SymbolToTable(cname);
end;

{ Boxed }

procedure Boxed.AddChild(n: Node);
begin
  if n is Field then
    begin fields.Add(n); if Length(n.cname) > maxfieldlength then maxfieldlength := Length(n.cname) end
  else if n is Method then
    methods.Add(n)
  else if n is _Constructor then
    constructors.Add(n)
  else if parent <> nil then parent.AddChild(n);
end;

constructor Boxed.Create(aParent: Node);
begin
  inherited;
  fields := TObjectList.Create(true);
  methods := TObjectList.Create(true);
  constructors := TObjectList.Create(true);
end;

destructor Boxed.Destroy;
begin
  fields.Free;
  methods.Free;
  constructors.Free;
  inherited;
end;

procedure Boxed.GenerateText;
var
  i: integer;
  f: Field;
begin
  Ln;
  EmitLn('(* ' + cname + ' boxed *)');
  Ln;
  Emit(Ident + cname + '* = RECORD');
  SymbolToTable(cname);
  if anSettings.target_language = tlCP then begin
    Emit(' [noalign]');
  end;
  Ln;
  for i := 0 to fields.Count - 1 do begin
    f := Field(fields[i]);
    EmitLn(Ident+Ident + f.cname + '*' + StringOfChar(' ', maxfieldlength - length(f.cname)) + ': ' + Lookup(f.typ) + ';');
  end;
  EmitLn(Ident + 'END;');
end;

procedure Boxed.parseAttr(a: TAttrList);
begin
  inherited;
  hidden  := boolvalue('hidden', a);
  opaque  := boolvalue('opaque', a);
end;

{ ReturnType }

procedure ReturnType.AddChild(n: Node);
begin
  (* do nothing *)
end;

procedure ReturnType.parseAttr(a: TAttrList);
begin
  inherited;
  owned          := boolvalue('owned', a);
  element_type   := strvalue('element_type', a);
  elements_owned := boolvalue('elements_owned', a);
end;

{ Callback }

procedure Callback.AddChild(n: Node);
begin
  if n is ReturnType then
    rettype := (n as ReturnType)
  else
    if n is Parameters then
      params := n as Parameters
    else
      if parent <> NIL then parent.AddChild(n);
end;

procedure Callback.GenerateCallbackDef(maxlength: integer);
var
  i: integer;
  p: Parameter;
  maxparamlen: integer;
begin
  Emit(Ident + cname + '*' + StringOfChar(' ', maxlength - Length(cname)) + ' = PROCEDURE ');
  DoEmitCdecl;
  Ln;
  Emit(Ident+'(');
  SymbolToTable(cname);

  if params <> nil then begin
    Ln;
    maxparamlen := 0;

    for i := 0 to params.paramlist.Count - 1 do begin
      p := Parameter(params.paramlist[i]);
      if Length(p.name) > maxparamlen then maxparamlen := Length(p.name)
    end;

    for i := 0 to params.paramlist.Count - 1 do begin
      p := Parameter(params.paramlist[i]);
      Emit(Ident+Ident+p.name + StringOfChar(' ', maxparamlen - Length(p.name)) + ': ' + Lookup(p.typ));
      if i = params.paramlist.Count - 1 then begin
        Ln;
      end ELSE begin
        EmitLn(';');
      end;
    end;
    Emit(Ident);
  end;

  Emit(')');
  if rettype <> nil then begin
    if rettype.typ <> 'void' then begin
      Emit(': ' + Lookup(rettype.typ))
    end;
  end;
  EmitLn(';');
  Ln;
end;

procedure Callback.GenerateText;
var
  i: integer;
  p: Parameter;
begin
  Emit(Ident + cname + '* = PROCEDURE (');
  if params <> nil then begin
    for i := 0 to params.paramlist.Count - 1 do begin
      p := Parameter(params.paramlist[i]);
      if i > 0 then Emit('; ');
      Emit(p.name + ': ' + p.typ);
    end;
  end;
  Emit(')');
  if rettype <> nil then begin
    Emit(': ' + rettype.typ)
  end;
  EmitLn(';');
end;

{ Parameter }

procedure Parameter.AddChild(n: Node);
begin
  (* do nothing *)
end;

procedure Parameter.parseAttr(a: TAttrList);
begin
  inherited;
  pass_as            := strvalue('pass_as', a);
  property_name      := strvalue('property_name', a);
  scope              := strvalue('scope', a);
  isarray            := boolvalue('array', a);
  ellipsis           := boolvalue('ellipsis', a);
  printf_format_args := boolvalue('printf_format_args', a);
  owned              := boolvalue('owned', a);
  printf_format      := boolvalue('printf_format', a);
end;

{ Namespace }

procedure Namespace.AddChild(n: Node);
begin
  if n is Callback then
    begin callbacs.Add(n); if Length(n.cname) > maxcallbacklength then maxcallbacklength := Length(n.cname) end
  else if n is Alias then
    begin aliases.Add(n); if Length(n.cname) > maxaliaslength then maxaliaslength := Length(n.cname) end
  else if n is Boxed then
    begin boxeds.Add(n); if Length(n.cname) > maxboxedlength then maxboxedlength := Length(n.cname) end
  else if n is Enum then
    enums.Add(n)
  else if n is _Interface then
    begin interfaces.Add(n); if Length(n.cname) > maxinterfacelength then maxinterfacelength := Length(n.cname) end
  else if n is _Object then
    begin objects.Add(n); if Length(n.cname) > maxobjectlength then maxobjectlength := Length(n.cname) end
  else if n is Struct then
    begin structs.Add(n); if Length(n.cname) > maxstructlength then maxstructlength := Length(n.cname) end
  else if n is _Class then
    classes.Add(n)
  else
    if parent <> nil then parent.AddChild(n);
end;

constructor Namespace.Create(aParent: Node);
begin
  inherited;
  enums      := TObjectList.Create(true);
  aliases    := TObjectList.Create(true);
  objects    := TObjectList.Create(true);
  classes    := TObjectList.Create(true);
  structs    := TObjectList.Create(true);
  callbacs   := TObjectList.Create(true);
  interfaces := TObjectList.Create(true);
  boxeds     := TObjectList.Create(true);
end;

destructor Namespace.Destroy;
begin
  enums      . Free;
  aliases    . Free;
  objects    . Free;
  classes    . Free;
  structs    . Free;
  callbacs   . Free;
  interfaces . Free;
  boxeds     . Free;
  inherited;
end;

procedure SortObjects(objects: TObjectList; null_parent: TObjectList; var nls: array of integer);
var
  sl: TStringList;
  o: _Object;
  cnt, i, j, t: integer;
  index: integer;
begin
  {    , .. ,    - ,
      ,         ,
         null_parent.  ,   
      ,     null_parent. }
  sl := TStringList.Create;
  for i := 0 to objects.Count - 1 do begin
    sl.AddObject(Node(objects[i]).cname, objects[i]);
  end;
  sl.Sorted := true;
  // sl.SaveToFile('sortedobjectlist');
  for i := 0 to objects.Count - 1 do begin
    o := _Object(objects[i]);
    if sl.Find(o.objparent, index) then
      _Object(sl.objects[index]).AddExtender(o)
    else
      null_parent.add(o);
    {end;}
  end;
  sl.Free;
  cnt := null_parent.Count;
  for i := 0 to cnt - 1 do
    nls[i] := i;

  {   null_parent   . }
  for i := 0 to cnt - 1 do begin
    for j := 1 to cnt - 1 do begin
      if _Object(null_parent[nls[j]]).children.Count >
         _Object(null_parent[nls[j-1]]).children.Count then begin

        //Writeln('SWAP ' + _Object(null_parent[j]).cname + ' & ' + _Object(null_parent[j-1]).cname);

        t := nls[j];
        nls[j] := nls[j-1];
        nls[j-1] := t;
      end;
    end;
  end;
end { SortObjects };

procedure GenerateObjectSubtree(o: _Object; sl, proc: TStringList);
var
  i: integer;
  c: _Object;
begin
  {      ,  ,   ,
         . }
  o.SortChildren;
  for i := 0 to o.children.Count - 1 do begin
    c := _Object(o.children[o.nls[i]]);
    c.GenerateText;
    c.AppendTextTo(sl);
    c.AppendProcTo(proc);
  end;
  for i := 0 to o.children.Count - 1 do begin
    c := _Object(o.children[o.nls[i]]);
    if c.children.Count > 0 then begin
      GenerateObjectSubtree(c, sl, proc);
    end;
  end;
end { GenerateObjectSubtree };

procedure Namespace.GenerateText;
const
  node_count = 8;
var
  si: array of Node;
  i: integer;
  null_parent: TObjectList;
  o: _Object;
  s: Struct;
  a: Alias;
  e: Enum;
  c: Callback;
  b: Boxed;
  nls: array of integer;
  ifs: _Interface;
begin
  if enums.Count > 0 then begin
    Ln;
    EmitLn('(* ==================================================================== ENUMS *)');
    for i := 0 to enums.Count - 1 do begin
      e := Enum(enums[i]);
      e.GenerateText;
      e.AppendTextTo(text);
    end;
    Ln;
  end;

  if objects.Count > 0 then begin
    EmitLn('(* ====================================================== POINTERS TO OBJECTS *)');
    EmitLn('TYPE');
    for i := 0 to objects.Count - 1 do begin
      o := _Object(objects[i]);
      EmitLn(ident + 'P' + o.cname + '*' + StringOfChar(' ', maxobjectlength - Length(o.cname)) + ' = POINTER TO ' + o.cname + ';');
      SymbolToTable('P' + o.cname);
    end;
    Ln;
  end;

  if structs.Count > 0 then begin
    EmitLn('(* ====================================================== POINTERS TO STRUCTS *)');
    EmitLn('TYPE');
    for i := 0 to structs.Count - 1 do begin
      s := Struct(structs[i]);
      EmitLn(ident + 'P' + s.cname + '*' + StringOfChar(' ', maxstructlength - Length(s.cname)) + ' = POINTER TO ' + s.cname + ';');
      SymbolToTable('P' + s.cname);
    end;
    Ln;
  end;

  if aliases.Count > 0 then begin
    EmitLn('(* ====================================================== POINTERS TO ALIASES *)');
    EmitLn('TYPE');
    for i := 0 to aliases.Count - 1 do begin
      a := Alias(Aliases[i]);
      EmitLn(Ident + 'P' + a.cname + '*' + StringOfChar(' ', maxaliaslength - Length(a.cname)) + ' = POINTER TO ' + a.cname + ';');
      SymbolToTable('P' + a.cname);
    end;
    Ln;
  end;

  if boxeds.Count > 0 then begin
    EmitLn('(* ======================================================= POINTERS TO BOXEDS *)');
    EmitLn('TYPE');
    for i := 0 to boxeds.Count - 1 do begin
      b := Boxed(boxeds[i]);
      EmitLn(Ident + 'P' + b.cname + '*' + StringOfChar(' ', maxboxedlength - Length(b.cname)) + ' = POINTER TO ' + b.cname + ';');
      SymbolToTable('P' + b.cname);
    end;
    Ln;
  end;

  if interfaces.Count > 0 then begin
    EmitLn('(* =================================================== POINTERS TO INTERFACES *)');
    EmitLn('TYPE');
    for i := 0 to interfaces.Count - 1 do begin
      ifs := _Interface(interfaces[i]);
      EmitLn(Ident + 'P' + ifs.cname + '*' + StringOfChar(' ', maxinterfacelength - Length(ifs.cname)) + ' = POINTER TO ' + ifs.cname + ';');
      SymbolToTable('P' + ifs.cname);
    end;
    Ln;
  end;


  if aliases.Count > 0 then begin
    EmitLn('(* ================================================================== ALIASES *)');
    EmitLn('TYPE');
    for i := 0 to aliases.Count - 1 do begin
      a := Alias(Aliases[i]);
      EmitLn(Ident + a.cname + '*' + StringOfChar(' ', maxaliaslength - Length(a.cname)) + ' = ' + Lookup(a.typ) + ';');
      SymbolToTable(a.cname);
    end;
    Ln;
  end;

  if callbacs.Count > 0 then begin
    Ln;
    EmitLn('(* ================================================================ CALLBACKS *)');
    EmitLn('TYPE');
    for i := 0 to callbacs.Count - 1 do begin
      c := Callback(callbacs[i]);
      c.GenerateCallbackDef(maxcallbacklength);
      c.AppendTextTo(text);
      //if text[text.Count-1] = '' then text.Delete(text.Count-1);
    end;
    Ln;
  end;

  if structs.Count > 0 then begin
    EmitLn('(* ================================================================== STRUCTS *)');
    EmitLn('TYPE');
    for i := 0 to structs.Count - 1 do begin
      s := Struct(structs[i]);
      s.GenerateText;
      s.AppendTextTo(text);
    end;
    Ln;
  end;

  if boxeds.Count > 0 then begin
    EmitLn('(* =================================================================== BOXEDS *)');
    EmitLn('TYPE');
    for i := 0 to boxeds.Count - 1 do begin
      b := Boxed(boxeds[i]);
      b.GenerateText;
      b.AppendTextTo(text);
    end;
    Ln;
  end;

  null_parent := TObjectList.Create;
  SetLength(nls, 10000);
  {      }
  SortObjects(objects, null_parent, nls);

//  Writeln('After sort');
//  for i := 0 to null_parent.Count - 1 do begin
//    o := _Object(null_parent[nls[i]]);
//    Writeln(o.cname + ', children count: ' + IntToStr(o.children.Count));
//  end;

  EmitLn('(*  ================================================================= OBJECTS *)');
  EmitLn('TYPE');
  for i := 0 to null_parent.Count - 1 do begin
    o := _Object(null_parent[nls[i]]);
    o.GenerateText;
    o.AppendTextTo(text);
    o.AppendProcTo(proc);
  end;

  for i := 0 to null_parent.Count - 1 do begin
    o := _Object(null_parent[i]);
//    Writeln(o.cname);
//    Writeln('children count: ' + IntToStr(o.children.Count));
    if o.children.Count > 0 then begin
      GenerateObjectSubtree(o, text, proc);
    end;
  end;
  Ln;

  if interfaces.Count > 0 then begin
    EmitLn('(* ============================================================= INTERFACES *)');
    EmitLn('TYPE');
    for i := 0 to interfaces.Count - 1 do begin
      ifs := _Interface(interfaces[i]);
      ifs.GenerateText;
      ifs.AppendTextTo(text);
      ifs.AppendProcTo(proc);
    end;
    Ln;
  end;


  SetLength(si, 0);
  null_parent.Free;
end;

procedure Namespace.parseAttr(a: TAttrList);
begin
  inherited;
  alibrary := strvalue('library', a);
end;

{ Api }

procedure Api.AddChild(n: Node);
begin
  if n is Namespace then
    namespaces.Add(n)
  else if n is Symbol then
    symbols.Add(n)
  else Utils.EmitLn('unknown type: <' + n.ClassName + '> name = "' + n.name + '" cname = "' + n.cname + '" type = "' + n.typ + '"')
end;

constructor Api.Create(aParent: Node);
begin
  inherited;
  parent := nil;
  symbols := TObjectList.Create(true);
  namespaces := TObjectList.Create(true);
end;

destructor Api.Destroy;
begin
  symbols.Free;
  namespaces.Free;
  inherited;
end;

procedure Api.GenerateText;
var
  i: integer;
  n: Namespace;
begin
  Emit('MODULE ');
  if anSettings.target_language = tlCP then begin
    Emit(anSettings.subsystem);
  end;
  Emit(anSettings.module_name);
  if (anSettings.lib <> '') and (anSettings.target_language = tlCP) then begin
    Emit(' ["' + anSettings.lib + '"]');
  end;
  EmitLn(';');
  if anSettings.target_language = tlXDS then begin
    EmitLn('<* + ALIGNMENT = "1" *>');
  end;
  for i := 0 to namespaces.Count-1 do begin
    n := Namespace(namespaces[i]);
    n.GenerateText;
  end;

  Ln;
  EmitLn('IMPORT');
  Emit(Ident+'SYSTEM');

  for i := 0 to stlst.Count - 1 do begin
    if stlst[i].isused and (not stlst[i].common) then begin
      EmitLn(',');
      Emit(Ident+stlst[i].TableName);
    end;
  end;
  EmitLn(';');
  Ln;

  for i := 0 to namespaces.Count-1 do begin
    n := Namespace(namespaces[i]);
    text.AddStrings(n.text);
    proc.AddStrings(n.proc);
  end;

  SwitchTo(proc);
  if anSettings.target_language = tlXDS then begin
    EmitLn('<* - ALIGNMENT = "1" *>');
  end;

  Emit('END ');
  if anSettings.target_language = tlCP then begin
    Emit(anSettings.subsystem);
  end;
  EmitLn(anSettings.module_name + '.');
end;

{ Parameters }

procedure Parameters.AddChild(n: Node);
begin
  if n is Parameter then
    paramlist.Add(n)
  else
    parent.AddChild(n);
end;

constructor Parameters.Create(aParent: Node);
begin
  inherited;
  paramlist := TObjectList.Create(true);
end;

destructor Parameters.Destroy;
begin
  paramlist.Free;
  inherited;
end;

{ Member }

procedure Member.AddChild(n: Node);
begin
  (* do nothing *)
end;

procedure Member.GenerateText;
begin

end;

procedure Member.parseAttr(a: TAttrList);
begin
  inherited;
  value := strvalue('value', a);
end;

{ Enum }

procedure Enum.AddChild(n: Node);
begin
  if n is Member then
    begin memberlist.Add(n); if Length(n.cname) > maxmemberlength then maxmemberlength := Length(n.cname); end
  else
    if parent <> nil then parent.AddChild(n);
end;

constructor Enum.Create(aParent: Node);
begin
  inherited;
  memberlist := TObjectList.Create(true);
end;

destructor Enum.Destroy;
begin
  memberlist.Free;
  inherited;
end;

procedure Enum.GenerateText;
var
  i: integer;
  m: Member;
  v: string;
  prevvalue: string;
  prevvalueidx: integer;
begin
  EmitLn('TYPE');
  Emit(Ident + cname + '* = ');
  SymbolToTable(cname);
  if typ = 'flags' then
    Emit('SET')
  else
    Emit(Lookup('gint32'));
 {end;}
  EmitLn(';');
  EmitLn('CONST');
  prevvalue := '';
  prevvalueidx := -1;
  for i := 0 to memberlist.Count - 1 do begin
    m := Member(memberlist[i]);
    if (m.value = '') then begin
        if (typ = 'flags') then
          v := '{}'
        else
          if (typ = 'enum') then
            v := IntToStr(i)
          else
            v := '<UNKNOWN>';
         {end;}
    end else begin
      v := m.value;
      prevvalue := v;
    end;
 (* end; *)

    (*   (a0 = xxx, a1, a2...) *)
    if (i > 0) and (prevvalue <> '') and (m.value = '') and (typ = 'enum') then begin
      v := prevvalue + '+' + IntToStr(i - prevvalueidx);
    end ELSE begin
      (* prevvalue := value; *)
      prevvalueidx := i;
    end;
    if m.cname <> '' then begin
      EmitLn(Ident + m.cname + '* ' + StringOfChar(' ', maxmemberlength - Length(m.cname)) + '= ' + Lookup(v) + ';');
      SymbolToTable(m.cname);
    end;
  end;
  Ln;
  SwitchTo(proc);
//  EmitLn(Ident + 'PROCEDURE ' +
end;

procedure Enum.parseAttr(a: TAttrList);
begin
  inherited;
  _deprecated := boolvalue('deprecated', a);
  gtype := strvalue('gtype', a);
  hidden := boolvalue('hidden', a);
end;

{ _Interface }

procedure _Interface.AddChild(n: Node);
begin
  if n is Signal then
    signallist.Add(n)
  else if n is Method then
    methodlist.Add(n)
  else if n is vmethod then
    vmethodlist.Add(n)
  else if n is _Property then
    properties.Add(n)
  else if parent <> nil then parent.AddChild(n);
end;

constructor _Interface.Create(aParent: Node);
begin
  inherited;
  properties := TObjectList.Create(true);
  signallist := TObjectList.Create(true);
  methodlist := TObjectList.Create(true);
  vmethodlist:= TObjectList.Create(true);
end;

destructor _Interface.Destroy;
begin
  properties  . Free;
  signallist  . Free;
  methodlist  . Free;
  vmethodlist . Free;
  inherited;
end;

procedure _Interface.GenerateText;
var
  i: integer;
  m: Method;
begin
  SwitchTo(text);
  Ln;
  EmitLn('(* ' + cname + ' interface *)');
  Ln;
  SymbolToTable(cname);

  Emit(ident + cname + '* = EXTENSIBLE RECORD');
  if anSettings.target_language = tlCP then begin
    Emit(' [noalign] ')
  end;
  Ln;
  EmitLn(Ident+'END;');

  (*   *)
  SwitchTo(proc);

  if methodlist.Count > 0 then begin
    Ln;
    EmitLn('(* ' + cname + ' methods *)');
    for i := 0 to methodlist.Count - 1 do begin
      m := Method(methodlist[i]);
      m.GenerateText;
      m.AppendProcTo(proc);
    end;
  end;
end;

{ VMethod }

procedure VMethod.AddChild(n: Node);
begin
  if n is Parameters then
    params := n as Parameters
  else if n is ReturnType then
    rettype := n as ReturnType
  else if parent <> nil then parent.AddChild(n);
end;

{ Signal }

procedure Signal.AddChild(n: Node);
begin
  if n is Parameters then
    params := n as Parameters
  else if n is ReturnType then
    rettype := n as ReturnType
  else if parent <> nil then parent.AddChild(n);
end;

procedure Signal.GenerateText;
var
  i, c: integer;
  p: Parameter;
begin
  if params = nil then Exit;
  if params.paramlist = nil then Exit;

  text.Clear;

  EmitLn(cname);
  Emit('*: PROCEDURE ');
  (* ccall *)
  Emit('(');
  c := params.paramlist.Count;
  for i := 0 to c - 1 do begin
    p := params.paramlist [i] as Parameter;
    if (p.pass_as = 'out') or (p.pass_as = 'ref') then begin
      Emit('VAR ');
    end;
    Emit(p.name + ': ' + p.typ);
    if i <> c - 1 then Emit('; ');
  end;
  Emit(')');
  if rettype.typ <> 'void' then begin
    Emit(': ' + rettype.typ);
  end;
  Emit(';');
end;

procedure Signal.parseAttr(a: TAttrList);
begin
  inherited;
  hidden := boolvalue('hidden', a);
  when   := strvalue('when', a);
end;

{ Method }

procedure Method.AddChild(n: Node);
begin
  if n is Parameters then
    params := n as Parameters
  else if n is ReturnType then
    rettype := n as ReturnType
  else if parent <> nil then parent.AddChild(n);
end;

procedure Method.GenerateText;
var
  i: integer;
  p: Parameter;
begin
  SwitchTo(proc);
  if (not anSettings.show_deprecated) and _deprecated then Exit;
  Emit(Ident + 'PROCEDURE ');

  DoEmitCdecl;

  Emit(' ' + cname + '*(');
  if not shared then begin
    Emit(LowerCase(parent.name) + ': P' + parent.cname);
    if (Params <> nil) and (params.paramlist.Count > 0) then Emit('; ');
  end;
  if params <> nil then begin
    for i := 0 to params.paramlist.Count - 1 do begin
      p := Parameter(params.paramlist[i]);
      if not p.ellipsis then begin
        if i > 0 then Emit('; ');
        Emit(p.name + ': ' + Lookup(p.typ));
      end;
    end;
  end;
  Emit(')');
  if rettype <> nil then begin
    if rettype.typ <> 'void' then begin
      Emit(': ' + Lookup(rettype.typ))
    end;
  end;
  EmitLn(';');
  FlushBuffer;
end;

procedure Method.parseAttr(a: TAttrList);
begin
  inherited;
  _deprecated := boolvalue('deprecated', a);
  hidden      := boolvalue('hidden', a);
  shared      := boolvalue('shared', a);
  needs_ref   := boolvalue('needs_ref', a);
end;

{ _Property }

procedure _Property.AddChild(n: Node);
begin
  (* do nothing *)
end;

procedure _Property.parseAttr(a: TAttrList);
begin
  inherited;
  construct_only := boolvalue('construct-only', a);
  readable    := boolvalue('readable', a);
  writeable   := boolvalue('writeable', a);
  construct   := boolvalue('construct', a);
  _deprecated := boolvalue('deprecated', a);
  hidden      := boolvalue('hidden', a);
  new_flag    := boolvalue('new_flag', a);
end;

{ _Object }

procedure _Object.AddChild(n: Node);
var
  s: string;
begin
  if n is Field then begin
      fields.Add(n);
      s := n.cname;
      if Field(n).bits <> '' then begin
        //     .
        // ,     flag   ( 10    ).
        //   "".
        // .
        s := 'flag0';
      end;
      if Length(s) > maxfieldlength then maxfieldlength := Length(s)
  end
  else if n is _Property then
    properties.Add(n)
  else if n is Method then
    methods.Add(n)
  else if n is Signal then
    signals.Add(n)
  else if n is _Constructor then
    constructors.Add(n)
  else if n is VMethod then
    vmethods.Add(n)
  else if n is Implements then
    implement := n as Implements
  else if n is Childprop then
    childprops.Add(n)
  else if n is StaticString then
    staticstrings.Add(n)
  else
    if parent <> nil then parent.AddChild(n);
end;

procedure _Object.AddExtender(n: Node);
begin
  children.Add(n);
end;

constructor _Object.Create(aParent: Node);
begin
  inherited;
  fields       := TObjectList.Create(true);
  properties   := TObjectList.Create(true);
  constructors := TObjectList.Create(true);
  methods      := TObjectList.Create(true);
  signals      := TObjectList.Create(true);
  vmethods     := TObjectList.Create(true);
  childprops   := TObjectList.Create(true);
  staticstrings:= TObjectList.Create(true);

  children     := TObjectList.Create;
end;

destructor _Object.Destroy;
begin
  fields       . Free;
  properties   . Free;
  constructors . Free;
  methods      . Free;
  signals      . Free;
  vmethods     . Free;
  childprops   . Free;
  staticstrings. Free;

  children     . Free;
  inherited;
end;

procedure _Object.GenerateText;
var
  i, j, k: integer;
  f: Field;
  m: Method;
  c: _Constructor;
  //   -  ,  -    
  bits: array of array of
    record
      bits,
      idlen: integer;
      cname: string;
    end;
  w1, w2: integer;
  maxbitfieldlength: integer;
  s: string;
begin
  SwitchTo(text);


  w1 := 0;
  w2 := 0;
  //     
  if fields.Count > 0 then begin
    //          
    for i := 0 to fields.Count - 1 do begin
      f := Field(fields[i]);
      //    
      if (i = 0) then begin
        if f.bits <> '' then begin
          SetLength(bits, w1 + 1);
          Inc(w1);
        end;
      end ELSE begin
        if (f.bits <> '') and (Field(fields[i-1]).bits = '') then begin
          SetLength(bits, w1 + 1);
          Inc(w1);
        end;
      end;

      //    ( )
      if (f.bits <> '') then begin
        SetLength(bits[w1 - 1], w2 + 1);
        Inc(w2);
        //  ,   ( ),  .
        with bits[w1-1, w2 - 1] do begin
          bits := StrToInt(f.bits);
          idlen := Length(f.cname);
          cname := f.cname;
        end;
      end ELSE begin
        w2 := 0;
      end;
    end;
  end;


  //     
  if w1 > 0 then begin
    Ln;
    EmitLn('(* Flags of ' + cname + ' object *)');
    Ln;
    EmitLn('CONST');
    i := 0;
    j := 0;
    maxbitfieldlength := 0;
    while i < w1 do begin
      while j < Length(bits[i]) do begin
        if Length(bits[i, j].cname) > maxbitfieldlength then maxbitfieldlength := Length(bits[i, j].cname);
        Inc(j);
      end;
      j := 0;
      Inc(i);
    end;
    i := 0;
    j := 0;
    k := 0;
    while i < w1 do begin
      while j < Length(bits[i]) do begin
        if bits[i, j].bits = 1 then begin
          s := IntToStr(k)
        end ELSE begin
          s := IntToStr(k) + '..' + IntToStr(k + bits[i, j].bits - 1);
        end;
        EmitLn(Ident + cname + '_' + 'flag' + IntToStr(i) + '_' + bits[i, j].cname + '_Mask*' +
           StringOfChar(' ', maxbitfieldlength - bits[i, j].idlen) + ' = {' + s + '};');
        k := k + bits[i, j].bits;
        Inc(j);
      end;
      j := 0;
      k := 0;
      Ln;
      Inc(i);
    end;
    Ln;
    EmitLn('TYPE');
  end;

  Ln;
  EmitLn('(* ' + cname + ' object *)');
  Ln;
  SymbolToTable(cname);

  Emit(ident + cname + '* = EXTENSIBLE RECORD');
  if anSettings.target_language = tlCP then begin
    Emit(' [noalign] ')
  end;
  if objparent <> '' then
    Emit('(' + Lookup(objparent) + ')');
  {end;}

  Ln;
//  if Length('parent') > maxfieldlength then maxfieldlength := Length('parent');
//  EmitLn(ident+ident+'parent*' +StringOfChar(' ', maxfieldlength - Length('parent')) + ': ' + Lookup(objparent) + ';');

  if Length(cname + '_flag0') > maxfieldlength then maxfieldlength := Length(cname + '_flag0');

  if fields.Count > 0 then begin
    k := 0;
    i := 0;
    while i < fields.Count do begin
      f := Field(fields[i]);
      if f.bits <> '' then begin
        while (i < fields.Count) and (Field(fields[i]).bits <> '') do begin
          Inc(i);
        end;
        EmitLn(Ident + Ident + cname + '_flag' + IntToStr(k) + '*' + StringOfChar(' ', maxfieldlength - Length(cname + '_flag' + IntToStr(k))) + ': SET;');
        Inc(k);
      end ELSE begin
        Emit(ident+ident);
        if (f.cname = 'private_data') or (f.cname = 'priv') then begin
          Emit(cname + '_pd*: ');
        end ELSE begin
          Emit(f.cname + '*' + StringOfChar(' ', maxfieldlength - Length(f.cname)) + ': ');
        end;
        if f._array then
          Emit('POINTER TO ARRAY [untagged] OF ');

        if (f.cname = 'private_data') or (f.cname = 'priv') then begin
          Emit(Lookup('gpointer') + ' (* private_data *)')
        end ELSE begin
          Emit(Lookup(f.typ));
        end;

        EmitLn(';');
      end;
      Inc(i);
    end;
  end;

  EmitLn(ident + 'END;');

  (*   *)
  SwitchTo(proc);

  if methods.Count > 0 then begin
    Ln;
    EmitLn('(* ' + cname + ' methods *)');
    for i := 0 to methods.Count - 1 do begin
      m := Method(methods[i]);
      m.GenerateText;
      m.AppendProcTo(proc);
    end;
  end;

  if constructors.Count > 0 then begin
    Ln;
    EmitLn('(* ' + cname + ' constructors *)');
    for i := 0 to constructors.Count - 1 do begin
      c := _Constructor(constructors[i]);
      c.GenerateText;
      c.AppendProcTo(proc);
    end;
  end;
end;

procedure _Object.parseAttr(a: TAttrList);
begin
  inherited;
  _deprecated := boolvalue('deprecated', a);
  disable_gtype_ctor := boolvalue('disable_gtype_ctor', a);
  objparent := strvalue('parent', a);
end;

procedure _Object.SortChildren;
var
  t, i, j, cnt: integer;
begin
  if children.Count = 0 then Exit;
  cnt := children.Count;
  SetLength(nls, cnt);
  for i := 0 to cnt - 1 do begin
    nls[i] := i;
  end;
  for i := 0 to cnt - 1 do begin
    for j := 1 to cnt - 1 do begin
      if _Object(children[nls[j]]).children.Count > _Object(children[nls[j-1]]).children.Count then begin
        t := nls[j];
        nls[j] := nls[j-1];
        nls[j-1] := t;
      end;
    end;
  end;
end;

{ Field }

procedure Field.AddChild(n: Node);
begin
  (* do nothing *)
end;

procedure Field.parseAttr(a: TAttrList);
begin
  inherited;
  access := strvalue('access', a);
  _array := boolvalue('array', a);
  array_len := strvalue('array_len', a);
  hidden := boolvalue('hidden', a);
  bits := strvalue('bits', a);
  writeable := boolvalue('writeable', a);
  new_flag := boolvalue('new_flag', a);
end;

{ _Constructor }

procedure _Constructor.AddChild(n: Node);
begin
  if n is Parameters then
    params := n as Parameters
  else if n is ReturnType then
    rettype := n as ReturnType
end;

constructor _Constructor.Create(aParent: Node);
begin
  inherited;

end;

destructor _Constructor.Destroy;
begin

  inherited;
end;

procedure _Constructor.GenerateText;
var
  i: integer;
  p: Parameter;
begin
  SwitchTo(proc);
  if (not anSettings.show_deprecated) and _deprecated then Exit;
  Emit(Ident + 'PROCEDURE ');

  DoEmitCdecl;
  Emit(' ' + cname + '*(');
  SymbolToTable(cname);
  if params <> nil then begin
    for i := 0 to params.paramlist.Count - 1 do begin
      p := Parameter(params.paramlist[i]);
      if i > 0 then Emit('; ');
      Emit(p.name + ': ' + Lookup(p.typ));
    end;
  end;
  Emit(')');
  if rettype <> nil then begin
    if rettype.typ <> 'void' then begin
      Emit(': ' + Lookup(rettype.typ))
    end;
  end ELSE begin
    Emit(': P' + parent.cname);
  end;
  EmitLn(';');
end;

procedure _Constructor.parseAttr(a: TAttrList);
begin
  inherited;
  _deprecated := boolvalue('deprecated', a);
  hidden := boolvalue('hidden', a);
  preferred := boolvalue('preferred', a);
  shared := boolvalue('shared', a);
end;

{ Implements }

procedure Implements.AddChild(n: Node);
begin
  if n is _Interface then
    interfaces.Add(n)
  else if parent <> nil then parent.AddChild(n);
end;

constructor Implements.Create(aParent: Node);
begin
  inherited;
  interfaces := TObjectList.Create(true);
end;

destructor Implements.Destroy;
begin
  interfaces.Free;
  inherited;
end;

{ Childprop }

procedure Childprop.AddChild(n: Node);
begin
  (* do nothing *)
end;

procedure Childprop.parseAttr(a: TAttrList);
begin
  inherited;
  readable := boolvalue('readable', a);
  writeable:= boolvalue('writeable', a);
end;

{ Struct }

procedure Struct.AddChild(n: Node);
var
  s: string;
begin
  if n is Field then
    begin fields.Add(n);
    s := n.cname;
    if Field(n).bits <> '' then begin
      //     .
      // ,     flag   ( 10    ).
      //   "".
      // .
      s := 'flag0';
    end;
    if Length(s) > maxfieldlength then maxfieldlength := Length(s) end
  else if n is Callback then
    callbacks.Add(n)
  else if n is Method then
    methods.Add(n)
  else if n is _Constructor then
    constructors.Add(n)
  else if parent <> nil then parent.AddChild(n);
end;

constructor Struct.Create(aParent: Node);
begin
  inherited;
  fields := TObjectList.Create(true);
  methods := TObjectList.Create(true);
  constructors := TObjectList.Create(true);
  callbacks := TObjectList.Create(true);
end;

destructor Struct.Destroy;
begin
  fields.Free;
  methods.Free;
  constructors.Free;
  callbacks.Free;
  inherited;
end;

procedure Struct.GenerateText;
var
  i, j, k: integer;
  s: string;
  w1, w2: integer;
  maxbitfieldlength: integer;
  f: Field;
  //   -  ,  -    
  bits: array of array of
    record
      bits,
      idlen: integer;
      cname: string;
    end;
  c: Callback;
  p: Parameter;
begin
  SwitchTo(text);

  w1 := 0;
  w2 := 0;
  //     
  if fields.Count > 0 then begin
    //          
    for i := 0 to fields.Count - 1 do begin
      f := Field(fields[i]);
      //    
      if (i = 0) then begin
        if f.bits <> '' then begin
          SetLength(bits, w1 + 1);
          Inc(w1);
        end;
      end ELSE begin
        if (f.bits <> '') and (Field(fields[i-1]).bits = '') then begin
          SetLength(bits, w1 + 1);
          Inc(w1);
        end;
      end;

      //    ( )
      if (f.bits <> '') then begin
        SetLength(bits[w1 - 1], w2 + 1);
        Inc(w2);
        //  ,   ( ),  .
        with bits[w1-1, w2 - 1] do begin
          bits := StrToInt(f.bits);
          idlen := Length(f.cname);
          cname := f.cname;
        end;
      end ELSE begin
        w2 := 0;
      end;
    end;
  end;


  //     
  if w1 > 0 then begin
    Ln;
    EmitLn('(* Flags of ' + cname + ' struct *)');
    Ln;
    EmitLn('CONST');
    i := 0;
    j := 0;
    maxbitfieldlength := 0;
    while i < w1 do begin
      while j < Length(bits[i]) do begin
        if Length(bits[i, j].cname) > maxbitfieldlength then maxbitfieldlength := Length(bits[i, j].cname);
        Inc(j);
      end;
      j := 0;
      Inc(i);
    end;
    i := 0;
    j := 0;
    k := 0;
    while i < w1 do begin
      while j < Length(bits[i]) do begin
        if bits[i, j].bits = 1 then begin
          s := IntToStr(k)
        end ELSE begin
          s := IntToStr(k) + '..' + IntToStr(k + bits[i, j].bits - 1);
        end;
        EmitLn(Ident + cname + '_' + 'flag' + IntToStr(i) + '_' + bits[i, j].cname + '_Mask*' +
           StringOfChar(' ', maxbitfieldlength - bits[i, j].idlen) + ' = {' + s + '};');
        k := k + bits[i, j].bits;
        Inc(j);
      end;
      j := 0;
      k := 0;
      Ln;
      Inc(i);
    end;
    Ln;
    EmitLn('TYPE');
  end;

//  
  Ln;
  EmitLn('(* ' + cname + ' struct *)');
  Ln;
  Emit(Ident + cname + '* = RECORD');

  SymbolToTable(cname);

  if anSettings.target_language = tlCP then begin
    EmitLn(' [noalign]');
  end ELSE begin
    Ln;
  end;

// 

  if fields.Count > 0 then begin
    k := 0;
    i := 0;
    while i < fields.Count do begin
      f := Field(fields[i]);
      if f.bits <> '' then begin
        while (i < fields.Count) and (Field(fields[i]).bits <> '') do begin
          Inc(i);
        end;
        EmitLn(Ident + Ident + 'flag' + IntToStr(k) + '*' + StringOfChar(' ', maxfieldlength - Length('flag' + IntToStr(k))) + ': SET;');
        Inc(k);
      end ELSE begin
        Emit(ident+ident);
        Emit(f.cname + '*' + StringOfChar(' ', maxfieldlength - Length(f.cname)) + ': ');
        if f._array then
          Emit('POINTER TO ARRAY [untagged] OF ');
        if f.cname = 'private_data' then Emit(Lookup('gpointer')) else Emit(Lookup(f.typ));
        EmitLn(';');
      end;
      Inc(i);
    end;
  end;
///    EmitLn(Ident + Ident + f.cname + '*' + StringOfChar(' ', maxfieldlength - length(f.cname)) + ': ' + Lookup(f.typ) + ';');
  k := 0;
  for i := 0 to callbacks.Count - 1 do begin
    c := Callback(callbacks[i]);
    if Length(c.cname) > k then k := Length(c.cname);
  end;

  for i := 0 to callbacks.Count - 1 do begin
    c := Callback(callbacks[i]);
    Emit(Ident+Ident + c.cname + '*' + StringOfChar(' ', k - Length(c.cname)) + ': PROCEDURE ');
    DoEmitCdecl;
    Emit(' (');
    SymbolToTable(cname);
    if c.params <> nil then begin
      for j := 0 to c.params.paramlist.Count - 1 do begin
        p := Parameter(c.params.paramlist[j]);
        if j > 0 then Emit('; ');
        Emit(p.name + ': ' + Lookup(p.typ));
      end;
    end;
    Emit(')');
    if c.rettype <> nil then begin
      if c.rettype.typ <> 'void' then begin
        Emit(': ' + Lookup(c.rettype.typ))
      end;
    end;
    EmitLn(';');
  end;

  EmitLn(Ident + 'END;');
end;

procedure Struct.parseAttr(a: TAttrList);
begin
  inherited;
  _deprecated := boolvalue('deprecated', a);
  hidden := boolvalue('hidden', a);
  opaque := boolvalue('opaque', a);
end;

{ _Class }

procedure _Class.AddChild(n: Node);
begin
  if n is Method then
    methods.Add(n)
  else if parent <> nil then parent.AddChild(n);
end;

constructor _Class.Create(aParent: Node);
begin
  inherited;
  methods := TObjectList.Create(true);
end;

destructor _Class.Destroy;
begin
  methods.Free;
  inherited;
end;

procedure _Class.parseAttr(a: TAttrList);
begin
  inherited;
  hidden := boolvalue('hidden', a);
end;

{ StaticString }

procedure StaticString.AddChild(n: Node);
begin
  (* do nothing *)
end;

procedure StaticString.parseAttr(a: TAttrList);
begin
  inherited;
  value := strvalue('value', a);
end;

{ Symbol }

procedure Symbol.AddChild(n: Node);
begin
  (* do nothing *)
end;

end.
