unit Settings;

interface

uses
  Classes, IniFiles;

type
  TargetLanguage = (tlCP, tlXDS);

  _Settings = class
    input_file: string;
    output_file: string;
    module_name: string;
    short_name: string;
    show_cdecl: boolean;
    target_language: TargetLanguage;
    show_deprecated: boolean;
    importList: TStringList;
    subsystem: string;
    lib: string;
    loaded: boolean;
    constructor Create;
    destructor Destroy; override;
  end;

procedure LoadSettings;

var
  anSettings: _Settings;

implementation

uses
  SysUtils;

{ Settings }

constructor _Settings.Create;
begin
  importList := TStringList.Create;
  loaded := false;
end;

destructor _Settings.Destroy;
begin
  importList.Free;
  inherited;
end;

const
  settings_section      = 'settings';

  input_file_param      = 'input file';
  output_file_param     = 'output file';
  module_name_param     = 'module name';
  short_name_param      = 'short name';
  subsystem_name_param  = 'subsystem';
  show_cdecl_param      = 'show cdecl';
  target_language_param = 'target language';
  import_list_param     = 'import list';
  library_param         = 'library';
  overwrite_param       = 'overwrite';

procedure LoadSettings;
var
  ini: TIniFile;
  s: string;
  inifilename: string;
begin
  inifilename := ExtractFilePath(ParamStr(0)) + '\' + ParamStr(1);
  if not FileExists(inifilename) then Exit;

  ini := TIniFile.Create(inifilename);

  anSettings.input_file  := ini.ReadString(settings_section, input_file_param, '');
  anSettings.output_file := ini.ReadString(settings_section, output_file_param, '');
  anSettings.show_cdecl  := ini.ReadBool(settings_section, show_cdecl_param, True);
  anSettings.module_name := ini.ReadString(settings_section, module_name_param, '');
  anSettings.short_name  := ini.ReadString(settings_section, short_name_param, '');
  anSettings.subsystem   := ini.ReadString(settings_section, subsystem_name_param, '');
  anSettings.lib         := ini.ReadString(settings_section, library_param, '');
  if UpperCase(ini.ReadString(settings_section, target_language_param, 'CP')) = 'XDS' then
    anSettings.target_language := tlXDS
  else
    anSettings.target_language := tlCP;
 {end;}

  s := ini.ReadString(settings_section, import_list_param, '');
  ExtractStrings([','], [], PChar(s), anSettings.importList);

  ini.Free;
  anSettings. loaded := true;
end;
initialization
  anSettings := _Settings.Create;
finalization
  anSettings.Free;
end.
