unit SymbolTables;

interface

uses
  Classes, Contnrs;

const
  symboltableext = '.st';

(*  ,    
       .

      .

      :
      <>.st,  <> -   ,  
        .

      :
      <>[,<SUBSYSTEM>][,COMMON]
             .
             ,
              .
            ,  .
               
       <> := <>,    IMPORT.

       
      <>[=<>]
      <>  ,    .
      [=<>] -   , ,
       1 << 1     {1},     
        
      1 << 1={1}

    ,      ()
        
             .

      
      :   (),   .
      :   ().

              ,    
         ,   .

                 
        ,       .

            ,     
        .

       .
      : 
      :  .
          ,   ,
      ..  <> := <>.
          ,   .

      
        ,   , ,  .

 *)

type
  Substitute = class
    value: string;
    constructor Create(val: string);
  end;

  SymbolTable = class
    common: boolean;
    shortname: string;
    modulename: string;
    subsystem: string;
    itemlist: TStringList;
    loaded: boolean;
    isused: boolean;
    procedure LoadFromFile(filename: string);
    function DoLookup(symbol: string): string;
    function TableName(): string;
    procedure AddSymbol(symbol: string);
    procedure SaveToFile;
    procedure Init();
    constructor Create;
    destructor Destroy; override;
  end;

  STList = class
  private
    function GetItems(Index: Integer): SymbolTable;
    procedure SetItems(Index: Integer; const Value: SymbolTable);
    function GetCount: integer;
  public
    list: TObjectList;
    property Count: integer read GetCount;
    function Lookup(id: string): string;
    procedure LoadTables;
    function Add(): SymbolTable;
    constructor Create;
    destructor Destroy; override;
    property Items[Index: Integer]: SymbolTable read GetItems write SetItems; default;
  end;

var
  stlst: STList;
  curst: SymbolTable;

implementation

uses
  SysUtils, Settings;

{ STList }

function STList.Add: SymbolTable;
begin
  Result := SymbolTable.Create;
  list.Add(Result);
end;

constructor STList.Create;
begin
  list := TObjectList.Create(true);
end;

destructor STList.Destroy;
begin
  list.Free;
  inherited;
end;

function STList.GetCount: integer;
begin
  Result := list.Count;
end;

function STList.GetItems(Index: Integer): SymbolTable;
begin
  Result := SymbolTable(list[index]);
end;

procedure STList.LoadTables;
var
  sr: TSearchRec;
  st: SymbolTable;
//  i: integer;
begin
  if FindFirst('*.st', faAnyFile, sr) = 0 then begin
    repeat
      //      .
      if not (ChangeFileExt(ExtractFileName(sr.Name), '') = anSettings.module_name) then begin
        st := stlst.Add;
        st.LoadFromFile(sr.Name);
      end;
      (*
      Writeln('==================================================================');
      Writeln(st.modulename);
      for i := 0 to st.itemlist.Count - 1 do begin
        if st.itemlist.objects[i] <> nil then
          Writeln(st.itemlist[i] + ' ' + Substitute(st.itemlist.objects[i]).value)
        else
          Writeln(st.itemlist[i]);
      end;*)
    until FindNext(sr) <> 0;
  end;
  FindClose(sr);
end;

function STList.Lookup(id: string): string;
var
  i: integer;
  t: string;
begin
  {        .
        . }
  Result := id;
  i := list.Count - 1;
  while i >= 0 do begin
    t := Items[i].DoLookup(id);
    if t <> '' then begin
      Result := t;
      exit;
    end;
    Dec(i)
  end;
end;

procedure STList.SetItems(Index: Integer; const Value: SymbolTable);
begin
  list[index] := value;
end;

{ SymbolTable }

procedure SymbolTable.AddSymbol(symbol: string);
begin
  itemlist.Add(symbol);
end;

constructor SymbolTable.Create;
begin
  itemlist := TStringList.Create;
  loaded := false;
  isused := false;
end;

destructor SymbolTable.Destroy;
begin
  itemlist.Free;
  inherited;
end;

function SymbolTable.DoLookup(symbol: string): string;
var
  i: integer;
begin
  i := itemlist.IndexOf(symbol);
  if i <> -1 then begin
    if itemlist.Objects[i] <> nil then begin
      if common then begin
        result := Substitute(itemlist.Objects[i]).value;
      end ELSE begin
        result := shortname + '.' + Substitute(itemlist.Objects[i]).value;
      end;
    end ELSE begin
      result := shortname + '.' + itemlist[i];
    end;
    isused := true;
  end ELSE begin
    result := '';
  end;
end;

procedure SymbolTable.Init;
begin
  common := false;
  shortname := anSettings.short_name;
  modulename := anSettings.module_name;
  subsystem := anSettings.subsystem;
  loaded := false;
  itemlist.Clear;
end;

procedure SymbolTable.LoadFromFile(filename: string);
var
  s: string;
  e: TStringList;
  i: integer;
begin
  if not FileExists(filename) then exit;
  itemlist.LoadFromFile(filename);
  if itemlist.Count = 0 then Exit;
  modulename := ChangeFileExt(ExtractFileName(filename), '');
  {    }
  s := itemlist[0];
  itemlist.Delete(0);
  e := TStringList.Create;
  {    }
  ExtractStrings([','], [' ', #09], PChar(s), e);
  {   }
  if e.Count = 0 then exit;
  shortname := e[0];
  {  }
  if e.Count > 1 then begin
    if UpperCase(e[1]) = 'COMMON' then
      common := true
    else
      subsystem := e[1];
  end;
  { COMMON }
  if e.Count > 2 then begin
    common := UpperCase(e[2]) = 'COMMON';
  end;

  {     }
  e.Clear;
  e.Assign(itemlist);
  itemlist.Clear;
  {    ,    = }
  for i := 0 to e.Count - 1 do begin
    if Pos('=', e[i]) <> 0 then begin
      itemlist.AddObject(e.Names[i], Substitute.Create(e.ValueFromIndex[i]))
    end ELSE begin
      itemlist.Add(e[i]);
    end;
  end;
  itemlist.Sorted := true;
  e.Free;
end;

procedure SymbolTable.SaveToFile;
var
  s: string;
begin
  s := shortname;
  if subsystem <> '' then begin
    s := s + ',' + subsystem;
  end;
  if common then begin
    s := s + ',COMMON';
  end;
  itemlist.Insert(0, s);
  itemlist.SaveToFile(modulename + symboltableext);
  itemlist.Delete(0);
end;

function SymbolTable.TableName: string;
begin
  if common then
    Result := ''
  else
    Result := shortname + ' := ' + subsystem + modulename;
 {end;}
end;

{ Substitute }

constructor Substitute.Create(val: string);
begin
  value := val;
end;

initialization
  stlst := STList.Create();
  curst := SymbolTable.Create;
finalization
  stlst.Free;
  curst.Free;
end.
