unit utils;

interface

uses
  Classes, SysUtils, Nodes, Nodes2, LibXmlParser;

var
  Ident: string = '  ';
  anApi: Api = nil;
  curnode: Node = nil;

procedure Emit(s: string);
procedure EmitLn(s: string);
procedure Ln;
procedure iEmitLn(identCount: integer; s: string);

procedure E2slLn(s: string; sl: TStringList);
procedure E2sl(s: string; sl: TStringList);
procedure Lnsl(sl: TStringList);
procedure iE2slLn(identCount: integer; s: string; sl: TStringList);

procedure PutDeclaration(name, value: string; exported: boolean; width: integer);
procedure PutProcedure(p: ProcNode; needCmodif: boolean; exported: boolean);
procedure PutCallback(c: CallbackNode; needCmodif: boolean; exported: boolean);

function ProcessTag(name: string; attr: TAttrList): Node;

procedure GenerateOutput;

{   ;      '*', 
           'p'. }
function UndecoratePtr(ptr: string): string;

implementation

uses SyntaxTree, Settings;

procedure Emit(s: string);
begin
  Write(s);
end;

procedure EmitLn(s: string);
begin
  WriteLn(s);
end;

procedure Ln;
begin
  Writeln;
end;

procedure iEmitLn(identCount: integer; s: string);
var
  i: integer;
begin
  for i := 0 to identCount - 1 do begin
    Emit(Ident);
  end;
  Emit(s);
  Ln;
end;

procedure PutDeclaration(name, value: string; exported: boolean; width: integer);
var
  r: string;
begin
  if name = '' then exit;
  r := Ident + name;
  if exported then r := r + '*';
  if width <> 0 then
    begin
      r := r + StringOfChar(' ', width - length(name));
    end;
  r := r + ' = ' + value + ';';
  EmitLn(r);
end;
procedure PutProcedure(p: ProcNode; needCmodif: boolean; exported: boolean);
begin
  if p = nil then Exit;
  if p.name = '' then Exit;
  Emit('PROCEDURE ');
  if needCmodif then Emit('[ccall] ');
  Emit(p.name);
  if exported then Emit('*');
  Emit('()');
  if p.ReturnType <> '' then Emit(': ' + p.ReturnType);
  Emit(';');
  Ln;
end;

procedure PutCallback(c: CallbackNode; needCmodif: boolean; exported: boolean);
var
  i: integer;
begin
  Emit('PROCEDURE ');
  if needCmodif then Emit('[ccall] ');
  Emit(c.cname);
  if exported then Emit('*');
  Emit('(');
  for i := 0 to c.paramList.Count - 1 do
    begin
      Emit(ParameterNode(c.paramList.Items[i]).name);
      Emit(': ');
      Emit(ParameterNode(c.paramList.Items[i]).atype);
      if (i <> c.paramList.Count-1) then Emit('; ');
    end;
  Emit(')');
  if c.return_type <> '' then Emit(': ' + c.return_type);
  (* after procedure *)
  Emit(';');
  Ln;
end;

function ProcessTag(name: string; attr: TAttrList): Node;
var
  n: Node;
begin
  n := nil;

  if      name = 'callback'       then n := Callback     . Create(curnode)
  else if name = 'boxed'          then n := Boxed        . Create(curnode)
  else if name = 'parameter'      then n := Parameter    . Create(curnode)
  else if name = 'namespace'      then n := Namespace    . Create(curnode)
  else if name = 'enum'           then n := Enum         . Create(curnode)
  else if name = 'member'         then n := Member       . Create(curnode)
  else if name = 'return-type'    then n := ReturnType   . Create(curnode)
  else if name = 'parameters'     then n := Parameters   . Create(curnode)
  else if name = 'signal'         then n := Signal       . Create(curnode)
  else if name = 'virtual_method' then n := VMethod      . Create(curnode)
  else if name = 'method'         then n := Method       . Create(curnode)
  else if name = 'property'       then n := _Property    . Create(curnode)
  else if name = 'interface'      then n := _Interface   . Create(curnode)
  else if name = 'field'          then n := Field        . Create(curnode)
  else if name = 'object'         then n := _Object      . Create(curnode)
  else if name = 'constructor'    then n := _Constructor . Create(curnode)
  else if name = 'implements'     then n := Implements   . Create(curnode)
  else if name = 'childprop'      then n := Childprop    . Create(curnode)
  else if name = 'struct'         then n := Struct       . Create(curnode)
  else if name = 'alias'          then n := Alias        . Create(curnode)
  else if name = 'class'          then n := _Class       . Create(curnode)
  else if name = 'static-string'  then n := StaticString . Create(curnode)
  else if name = 'symbol'         then n := Symbol       . Create(curnode)
  else EmitLn('unknown node: <' + name + '>');

  if n <> nil then begin
    n.parseAttr(attr);
    curnode.AddChild(n);
  end;
  Result := n;
end;

procedure E2slLn(s: string; sl: TStringList);
begin
  sl.Add(s);
end;

procedure E2sl(s: string; sl: TStringList);
begin
  sl[sl.Count-1] := sl[sl.Count-1] + s; 
end;

procedure Lnsl(sl: TStringList);
begin
  sl.Add('');
end;

procedure iE2slLn(identCount: integer; s: string; sl: TStringList);
var
  i: integer;
begin
  for i := 0 to identCount - 1 do begin
    E2sl(ident, sl);
  end;
  E2sl(s, sl);
  Lnsl(sl);
end;

function UndecoratePtr(ptr: string): string;
begin
  if ptr = '' then begin
    Result := '';
    exit;
  end;
  Result := ptr;
  if Result[Length(Result)] = '*' then
    begin
      SetLength(Result, Length(Result) - 1);
      Result := 'p'+Result;
    end;
end;

procedure GenerateOutput;
begin
  if anApi = nil then Exit;
  if anApi.namespaces.Count = 0 then Exit;
  anApi.GenerateText;
  anApi.text.AddStrings(anApi.proc);
  anApi.text.SaveToFile(anSettings.output_file);
end;

end.
