program gapiParser;
{$APPTYPE Console}
uses
  SysUtils,
  Classes,
  LibXmlParser,
  SyntaxTree,
  utils,
  KnownIdents,
  Nodes2 in 'Nodes2.pas',
  SymbolTables in 'SymbolTables.pas',
  Settings in 'Settings.pas';

procedure ShowUsage();
begin
  Writeln('GAPI parser. Parses GAPI xml files and produces Component Pascal/Oberon-2 modules.');
  Writeln('(c) 2009 Solo');
  Writeln;
  Writeln('Usage:');
  Writeln('  ' + ExtractFileName(ParamStr(0)) + ' <settings.ini>');
end;

procedure ParseParams();
var
  inifilename: string;
begin
  if (ParamCount = 0) or (ParamStr(1) = '/?') or (ParamStr(1) = '-?') or (ParamStr(1) = '?') then begin
    ShowUsage();
    Halt(0);
  end;
  inifilename := ExtractFilePath(ParamStr(0)) + '\' + ParamStr(1);
  if not FileExists(inifilename) then begin
    ShowUsage;
    Halt(1);
  end;
end;

procedure ParseFile(FileName: string);
var
  xml: TXMLParser;
  n: Node;
begin

  xml := TXmlParser.Create;
  xml.LoadFromFile (FileName);
  xml.StartScan;

  WHILE xml.Scan DO BEGIN
    CASE xml.CurPartType OF
      (*ptStartTag: begin Emit('ptStartTag: <' + xml.CurName + '>'); for i := 0 to xml.CurAttr.Count - 1 do Emit(' ' + xml.CurAttr.Name(i)); Ln() end;
      ptEmptyTag: begin Emit('ptEmptyTag: <' + xml.CurName + '>'); for i := 0 to xml.CurAttr.Count - 1 do Emit(' ' + xml.CurAttr.Name(i)); Ln() end;
      //ptEndTag  : begin Emit('ptEndTag  : ' + xml.CurName); for i := 0 to xml.CurAttr.Count - 1 do Emit(' ' + xml.CurAttr.Name(i)); Ln() end;
      ptXmlProlog: emitln('Prolog: ' + xml.CurName);*)
        (*
        BEGIN
          if xml.CurName = 'enum' then
            ProcessEnum(xml) else
          if xml.CurName = 'callback' then
            ProcessCallback(xml);
          if xml.CurName = 'interface' then
            ProcessInterface(xml);
    	  END;
        *)
      ptStartTag,
      ptEmptyTag: begin
        if xml.CurName = 'api' then
          if anApi = nil then begin
            anApi := Api.Create(nil);
            curnode := anApi;
          end else begin
            EmitLn('Error: <api> tag duplicated!. Exit...');
            Halt(1);
          end
        else
          begin
            if anApi = nil then begin
              EmitLn('Error: <api> tag expected!. Exit...');
              Halt(1);
            end ELSE begin
              n := ProcessTag(xml.CurName, xml.CurAttr);
              if (n<>nil) and (xml.CurPartType = ptStartTag) then begin
                curnode := n;
              end;
            end;
          end;
      end;
      ptEndTag: begin
        if (curnode <> nil) and (curnode.parent <> nil) then begin
          curnode := curnode.parent;
        end;
      end;
    END;
  END;
  xml.Free;
end;


begin
  ParseParams();
  Settings.LoadSettings;
  stlst.LoadTables;
  curst.Init;

  Writeln('Parameters ok. Processing ' + anSettings.input_file + '...');

  Writeln('Parsing file');
  ParseFile(anSettings.input_file);

  Writeln('Generating output');
  GenerateOutput;

  curst.SaveToFile;

  Writeln('Done');
end.
